#ifndef n_ByteOrder_H
#define n_ByteOrder_H

/*
 *
 * nucleo/utils/ByteOrder.H --
 *
 * Copyright (C) Nicolas Roussel
 *
 * See the file LICENSE for information on usage and redistribution of
 * this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdint.h>

#ifdef __APPLE__
#include <CoreFoundation/CoreFoundation.h>
#endif

namespace nucleo {

  namespace ByteOrder {

#ifdef __APPLE__
    bool isLittleEndian(void) {
	 return CFByteOrderGetCurrent()==CFByteOrderLittleEndian ;
    }
    uint16_t swap16ifle(uint16_t arg) {
	 return CFSwapInt16BigToHost(arg) ;
    }
    uint32_t swap32ifle(uint32_t arg) {
	 return CFSwapInt32BigToHost(arg) ;
    }
#else
    bool isLittleEndian(void) {
	 static uint32_t littleEndianTest = 1 ;
	 return (*(char *)&littleEndianTest == 1) ;
    }
    uint16_t swap16ifle(uint16_t arg) {
	 return isLittleEndian() ? ((((arg) & 0xff) << 8) | (((arg) >> 8) & 0xff)) : arg ;
    }
    uint32_t swap32ifle(uint32_t arg) {
	 return isLittleEndian() ? ((((arg) & 0xff000000) >> 24) |	\
						 (((arg) & 0x00ff0000) >> 8)  | \
						 (((arg) & 0x0000ff00) << 8)  | \
						   (((arg) & 0x000000ff) << 24))  : arg ;
    }
#endif

  }

}

#endif
